/*
 * SDRC register values for Nokia boards
 *
 * Copyright (C) 2010 Nokia
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

struct omap_sdrc_params *nokia_get_sdram_timings(void);

